#import "RadioRecorder.h"

RadioRecorder *recorder;

@implementation RadioRecorder
-(NSString *)prefPath {
	NSArray *paths = NSSearchPathForDirectoriesInDomains(NSLibraryDirectory,NSUserDomainMask, YES);
	return [[paths objectAtIndex:0] stringByAppendingPathComponent:@"RadioRecorder.reclist"];
}

-(void)loadRecList {
	[recList release];
	recList = [NSKeyedUnarchiver unarchiveObjectWithFile: [self prefPath]];
	if (recList == nil) {
		recList = [[NSMutableArray alloc] init];
		[self newRecord];
	} else {
	    [recList retain];
		[self loadRecord: 0];
	}
}

-(void)storeRecList {
	[NSKeyedArchiver archiveRootObject:recList toFile:[self prefPath]];
}

-(BOOL)hasRecordNamed: (NSString *)name {
	int i, len = [recList count];
	for (i = 0; i < len; i++) {
		Recording *r = [recList objectAtIndex:i];
		if ([name caseInsensitiveCompare: r->name] == NSOrderedSame)
			return YES;
	}
	return NO;
}

-(void)newRecord {
	int i;
	NSString *name;
	for (i = 1;;i++) {
		name = [NSString stringWithFormat:@"New Recording %d", i];
		if (![self hasRecordNamed: name])
			break;
	}
	Recording *r = [[Recording alloc] initWithName:name];
	[recList addObject: r];
	[r release];
	[self loadRecord: [recList count]-1];
}

- (IBAction)clonePressed:(id)sender  {
	if(![self saveRecord])
		return;
	int i;
	NSString *name;
	for (i = 2;;i++) {
		name = [current->name stringByAppendingFormat:@" %d", i];
		if (![self hasRecordNamed: name])
			break;
	}
	Recording *r = [[Recording alloc] initWithRecording:current name:name];
	[recList addObject: r];
	[r release];
	[self loadRecord: [recList count]-1];
}

+(void)set: (id)obj toHidden: (BOOL)state {
	if ([obj respondsToSelector:@selector(setHidden:)])
		[obj setHidden: state]; 
}

-(void)enableButtons {
	[dailyButton setEnabled: [onceButton state] == NSOffState];
	BOOL hasDays = ![dailyButton isEnabled] || [dailyButton state] == NSOffState;
	int i;
	for (i = 0; i < 7; i++)
		[days[i] setEnabled: hasDays];
	if (current->inProgress) {
		[nowButton setEnabled: false];
		//[saveButton setEnabled: false];
		[deleteButton setEnabled:false];
		[stopButton setEnabled: true];
		[RadioRecorder set: progress toHidden: NO];
	} else {
		[nowButton setEnabled: true];
		//[saveButton setEnabled: true];
		[deleteButton setEnabled: true];
		[stopButton setEnabled: false];
		[progress setDoubleValue:0];
		[RadioRecorder set: progress toHidden: YES];
	}
	bool toSplit = [splitButton state] == NSOnState;
	[shoutLang setEnabled:toSplit];
	[splitTitle setEnabled:toSplit];
	[keepPartial setEnabled:toSplit];
//	[RadioRecorder set: shoutLangLabel toHidden:!toSplit];
}
	
-(void)loadRecord: (int)idx {
	if (current)
		[current detach];
	curIndex = idx;
	current = [recList objectAtIndex: idx];
	[current loadToUI:self];
	// Exceptions
	int i;
	for (i = 0; i < sizeof(days)/sizeof(days[0]); i++)
		[days[i] setState: current->days[i]];
	[hourDuration setIntValue: current->duration/60];
	[minuteDuration setIntValue: current->duration%60];
	int h = current->startTime/60;
	if (h < 12)
		[ampm setStringValue: @"AM"];
	else {
		h-=12;
		[ampm setStringValue: @"PM"];
	}
	if (!h) h = 12;
	[hourTime setIntValue: h];
	[minuteTime setIntValue: current->startTime % 60];
	[shoutLang selectItemAtIndex:0];
	for (i = 0; langList[i] != kCFStringEncodingInvalidId; i++)
		if (current->shoutLang == langList[i]) {
			[shoutLang selectItemAtIndex:i];
			break;
		}
	[self enableButtons];
	[current attachLog: log progress: progress];
}

-(NSString *)trim:(NSString *)str {
	return [str stringByTrimmingCharactersInSet: [NSCharacterSet whitespaceCharacterSet]];
}

-(int)alertWithTitle: (NSString *)title Text: (NSString *)value {
	return NSRunAlertPanel(title, value, nil, nil, nil);
}

-(BOOL)saveRecord {
	NSString *name = [recordingChoice stringValue], *url = [stationField stringValue], *pl = [playlistField stringValue];
	name = [self trim: name]; url = [self trim: url]; pl = [self trim: pl];
	int nl = [name length];
	while (nl > 0 && [name characterAtIndex: nl-1] == '*')
		name = [name substringToIndex: --nl];
	name = [self trim: name];
	[recordingChoice setStringValue: name];
	if ([name caseInsensitiveCompare: current->name] != NSOrderedSame &&
		[self hasRecordNamed: name]) {
		[self alertWithTitle: @"Name already in use"
						Text: [NSString stringWithFormat: @"You already have a recording named %@. Please edit your entry.", name]];
		return NO;
	}
	if ([url length] == 0) {
		[self alertWithTitle: @"Missing Station URL" Text: @"Please fill in the station field"];
		return NO;
	}
	if ([NSURL URLWithString: url] == nil) {
		[self alertWithTitle:@"Invalid URL" Text:@"Station URL is invalid. Perhaps you should start it with http://?"];
		return NO;
	}
	[current saveFromUI:self];
	int ht = [hourTime intValue], mt = [minuteTime intValue], hd = [hourDuration intValue], md = [minuteDuration intValue];
	NSString *amst = [ampm stringValue];
	BOOL pm = toupper([amst characterAtIndex:0]) == 'P';
	if (ht == 12) ht = 0;
	if (pm) ht+=12;
	current->startTime = ht*60+mt;
	current->duration = hd*60+md;
	int i;
	for (i = 0; i < sizeof(days)/sizeof(days[0]); i++)
		current->days[i] = [days[i] state] == NSOnState;
	current->shoutLang = langList[[shoutLang indexOfSelectedItem]];
	if (!current->inProgress)
		[current schedule];
	[self storeRecList];
	return YES;
}

-(void)fixComboBox: (NSComboBox *)cb {
	// Can not use string values generated by InterfaceBuilder with NSNumberFormatter
	int cnt = [cb numberOfItems], i;
	for (i = 0; i < cnt; i++) {
		NSString *str = [cb itemObjectValueAtIndex: i];
		NSDecimalNumber *d = [NSDecimalNumber decimalNumberWithString:str];
		[cb removeItemAtIndex: i];
		[cb insertItemWithObjectValue:d atIndex:i];
	}
}

-(void)awakeFromNib {
	NSArray *typeArray = [NSArray arrayWithObjects: NSURLPboardType, nil];
	[window registerForDraggedTypes:typeArray];
	days[0] = monButton; days[1] = tueButton; days[2] = wedButton; days[3] = thuButton;
	days[4] = friButton; days[5] = satButton; days[6] = sunButton;
	[self fixComboBox: hourTime];
	[self fixComboBox: minuteTime];
	[self fixComboBox: hourDuration];
	[self fixComboBox: minuteDuration];
	[self fixComboBox: recCount];
	langList = CFStringGetListOfAvailableEncodings();
	const CFStringEncoding *p = langList;
	while (*p != kCFStringEncodingInvalidId) {
		CFStringRef n = CFStringGetNameOfEncoding(*p);
		if (n != nil)
			[shoutLang addItemWithObjectValue: (NSString *)n];
		p++;
	}
	recorder = self;
	[self loadRecList];
}

- (void)windowWillClose:(NSNotification *)aNotification {
	int i;
	for (i = 0; i < [recList count]; i++) {
		Recording *r = [recList objectAtIndex:i];
		[r endRecord];
	}
	[self saveRecord];
}

-(NSDragOperation)draggingEntered: (id<NSDraggingInfo>) sender {
	unsigned int mask = [sender draggingSourceOperationMask];
	unsigned int ret = NSDragOperationCopy & mask;
	return ret;
}

-(BOOL)prepareForDragOperation: (id<NSDraggingInfo>) sender {
	return YES;
}

-(BOOL)performDragOperation:(id <NSDraggingInfo>) sender {
	NSPasteboard *pb = [sender draggingPasteboard];
	NSArray *urlA = [pb propertyListForType:NSURLPboardType];
	NSString *url = [urlA objectAtIndex:0];
	NSString *key = [NSString stringWithFormat:@"CorePasteboardFlavorType 0x%X", 'urln'];
	NSString *urln = [pb stringForType:key];
	if (urln) {
		[recordingChoice setStringValue:urln];
		[playlistField setStringValue:urln];
	}
	[stationField setStringValue:url];
	return YES;
}


- (IBAction)dailyPressed:(id)sender
{
	[self enableButtons];
}

- (IBAction)deletePressed:(id)sender
{
	int r = NSRunAlertPanel(@"Delete this recording?",@"Do you want to proceed?", @"Cancel",@"Delete",nil);
	if (r == NSAlertDefaultReturn) return;
	[current detach];
	[current endRecord];
	[current deschedule];
	current = nil;
	[recList removeObjectAtIndex:curIndex];
	int len = [recList count];
	if (len == 0)
		[self newRecord];
	else {
		if (curIndex == len) curIndex--;
		[self loadRecord: curIndex];
	}
	[self storeRecList];
}

- (IBAction)newPressed:(id)sender
{
	if ([self saveRecord]);
	   [self newRecord];
}

- (IBAction)nowPressed:(id)sender
{
	if ([self saveRecord])
		[current record: nil];
}

- (IBAction)adjustEnabled:(id)sender
{
	[self enableButtons];
}

- (IBAction)recordingPicked:(id)sender
{
}

- (IBAction)savePressed:(id)sender
{
	[self saveRecord];
}

- (IBAction)stopPressed:(id)sender
{
	[current log: @"Stopped by user\n"];
	[current endRecord];
}

- (IBAction)weekdayPressed:(id)sender
{
}

- (IBAction)revertPressed:(id)sender {
	[self loadRecord: curIndex];
}

- (void)dealloc {
	[recList release];
	[saveName release];
}

- (void)comboBoxWillPopUp:(NSNotification *)notification {
	[recordingChoice removeAllItems];
	[recList sortUsingSelector:@selector(compareTo:)];
	for (curIndex=0; curIndex < [recList count]; curIndex++)
		if (current == [recList objectAtIndex:curIndex])
			break;
	NSAssert(curIndex < [recList count], @"Can't find record after sort");
	int len = [recList count], i;
	for (i = 0; i < len; i++) {
		Recording *r = [recList objectAtIndex:i];
		NSString *name = r->name;
		if (r->inProgress)
			name = [name stringByAppendingString: @" *"];
		[recordingChoice addItemWithObjectValue:name];
	}
}

- (void)comboBoxSelectionIsChanging:(NSNotification *)notification {
	[saveName release];
	saveName = [[recordingChoice stringValue] retain];
}

- (void)comboBoxSelectionDidChange:(NSNotification *)notification {
	int idx = [recordingChoice indexOfSelectedItem];
	if (idx == -1 || idx == curIndex) return;
	if ([self saveRecord] == NO)
		[recordingChoice setStringValue: saveName];
	else
		[self loadRecord: [recordingChoice indexOfSelectedItem]];
}
	

-(void)clearDay: (int)day {
	[days[day] setState: NSOffState];
}

-(IBAction)displayHelp: (id)sender {
	NSWorkspace *w = [NSWorkspace sharedWorkspace];
	NSString *s = [[NSBundle bundleForClass:[self class]] pathForResource:@"RadioRecorder" ofType:@"html" inDirectory:@"RadioRecorder Help"];
	[w openFile:s];
}
@end
