//
//  Recording.m
//  RadioRecorder
//
//  Created by Oleg Kibirev on Sun Apr 25 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "Recording.h"
#import "RadioRecorder.h"
#import "UIValueManager.h"
#import "NSAppleScript+HandlerCalls.h"

static UIValInfo RecordingUIVals[] = {
	{"name", "recordingChoice"},
	{"url", "stationField", "http://"},
	{"playList", "playlistField", "My Radio Recordings"},
	{"customTitle", "titleField"},
	{"customArtist", "artistField", "Radio Recorder"},
	{"customAlbum", "albumField"},
	{"customGenre", "genreChoice", "Radio Recorder"},
	{"daily", "dailyButton"},
	{"once", "onceButton"},
	{"recCount", "recCount", NULL, 1000},
	{"splitSongs", "splitButton"},
	{"reconnect", "reconnectButton"},
    {"splitTitle", "splitTitle", NULL, 1},
    {"keepPartial", "keepPartial"},
	{"startTime", NULL, NULL, 10*60},
	{"duration", NULL, NULL, 60},
	{"shoutLang", NULL, NULL, kCFStringEncodingUTF8}
};

#define ENDOF @"End of "
#define STARTOF @"Start of "

static UIValueManager *uiManager;
static NSAppleScript *addScript;

@implementation Recording

+(void)initialize {
	if (!uiManager)
		uiManager = [[UIValueManager alloc] initWithValInfo:RecordingUIVals ofCount:sizeof(RecordingUIVals)/sizeof(RecordingUIVals[0])];

}

-(void)encodeWithCoder:(NSCoder *)coder {
	[uiManager encodeObject:self withCoder:coder];
	[coder encodeBytes:days length:sizeof(days) forKey:@"days"];
	[coder encodeObject: log forKey:@"log"];
}

-(id)initWithCoder:(NSCoder *)coder {
	self = [super init];
	[uiManager decodeObject:self withCoder:coder];
	unsigned len;
	const void *p = [coder decodeBytesForKey:@"days" returnedLength:&len];
	memcpy(days, p, len);
	log = [[coder decodeObjectForKey: @"log"] retain];
	[self schedule];
	return self;
}

-(id)initWithName: (NSString *)n {
	self = [super init];
	[uiManager initObject:self];
	log = [[NSMutableString alloc] init];
	[n retain];
	[name release];
	name = n;
	return self;
}

-(id)initWithRecording: (Recording *)obj name: (NSString *)n {
	self = [super init];
	[uiManager copyObject:self fromObject:obj];
	memcpy(days, obj->days, sizeof(days));
	log = [[NSMutableString alloc] init];
	[n retain];
	[name release];
	name = n;
	return self;
}

-(void)loadToUI: (RadioRecorder *)rec {
	[uiManager loadObject:self toUI:rec];
}

-(void)saveFromUI: (RadioRecorder *)rec {
	[uiManager saveUI:rec toObject:self];
}

-(void)dealloc {
	[self endRecord];
	[self deschedule];
	[uiManager deallocObject:self];
	[log release];
	[startDate release];
	[filePath release];
	[icyData release];
	[icyLast release];
	[genre release];
	[title release];
	[nextDate release];
	[super dealloc];
}

-(void) attachLog: (NSTextView *)txt progress: (NSProgressIndicator *)prog {
	self->text = txt; self->progress = prog;
	[progress setDoubleValue: percent];
	[text setString: log];
	[text scrollRangeToVisible:NSMakeRange([log length]-1, 1)];
}

-(void) detach {
	text = nil; progress = nil;
}

-(void) log: (NSString *)txt {
	if ([log length] >= 16000)
		[log deleteCharactersInRange:NSMakeRange(0, 8000)];
	[log appendString: txt];
	if (text) {
		[text setString : log];
		[text scrollRangeToVisible:NSMakeRange([log length]-1, 1)];
	}
	
}

-(void) fmtlog: (NSString *)fmt, ... {
	va_list va;
	va_start(va, fmt);
	NSString *s = [[NSString alloc] initWithFormat:fmt arguments:va];
	va_end(va);
	[self log: s];
	[s release];
}

-(void) deschedule {
	if (timer) {
		[timer invalidate];
		[timer autorelease];
		timer = nil;
	}
}

-(NSString *)stringFromDate: (NSCalendarDate *)d {
	return [d descriptionWithCalendarFormat: 
		[[NSUserDefaults standardUserDefaults] stringForKey:NSTimeDateFormatString]];
}

-(NSString *)langString: (const void *)buf withLength: (int)len {
	NSString *s = (NSString *) CFStringCreateWithBytes(NULL, buf, len, shoutLang, NO);
	if (s == nil) // One more chance if specified encoding is incorrect
		s = [NSString stringWithCString:(const char *)buf length:len];
	else
		[s autorelease];
	return s;
}
				

-(void) schedule {
	[self deschedule];
	NSCalendarDate *cal = [NSCalendarDate calendarDate];
	int tim = [cal hourOfDay]*60+[cal minuteOfHour];
	cal = [NSCalendarDate dateWithYear: [cal yearOfCommonEra]
								 month: [cal monthOfYear] 
								   day: [cal dayOfMonth]
								  hour: 0 minute:0 second:0 
							  timeZone: [NSTimeZone defaultTimeZone]]; // Start of this day
	
	int cday = [cal dayOfWeek], d;
	cday = cday == 0 ? 6 : cday-1;
	BOOL dl = daily && !once;
	[nextDate release]; nextDate = nil;
	// Check for reconnect
	if (reconnect && willResume) {
		[self fmtlog:@"Will reconnect after 15 seconds\n"];
		nextDate = [[[NSCalendarDate calendarDate] dateByAddingYears:0 months:0 days:0 hours:0 minutes:0 seconds:15] retain];
	} else {
		for (d = startTime >= tim ? 0 : 1; d <= 7; d++)
			if (dl || days[(cday+d) % 7])
				break;
		if (d > 7) {
			[self log: @"The recording is disabled\n"];
			return;
		}
		nextDate = [[cal dateByAddingYears:0 months:0
									  days:d hours:0 minutes:startTime seconds:0] retain];
	}
	/*
	 * Scheduling is unreliable in case of sleep and wake up. Check every 5 minutes if the time is already reached
	 */
	[self fmtlog: @"Next recording is at %@\n", [self stringFromDate:nextDate]];
	[self checkTime: nil];
}

-(void) checkTime: (NSTimer *)tim {
	NSTimeInterval t = [nextDate timeIntervalSinceNow];
	if (t <= 0 && tim != nil) {
		[self record: tim];
		return;
	}
	[self deschedule];
	NSCalendarDate *nxtCheck;
	if (t <= 10*60)
		nxtCheck = nextDate;
	else
		nxtCheck = [[NSCalendarDate calendarDate] dateByAddingYears:0 months:0 days:0 hours:0 minutes:5 seconds:0];
	assert (timer == nil);
	timer = [[NSTimer alloc] initWithFireDate:nxtCheck interval: 0 target:self selector: @selector(checkTime:)
									 userInfo:nil repeats:NO];
	[[NSRunLoop currentRunLoop] addTimer:timer forMode:NSDefaultRunLoopMode];
}	


-(BOOL) connect: (NSString *)loc {
	[self fmtlog:@"Connecting to %@\n", loc];
	if (conn != nil) {
		[conn cancel];
		[conn autorelease];
		conn = nil;
	}
	NSURL *u = [NSURL URLWithString: loc];
	NSMutableURLRequest *req = [[NSMutableURLRequest alloc] initWithURL:u
															cachePolicy:  NSURLCacheStorageNotAllowed
														timeoutInterval: CONN_TIMEOUT];
	[req autorelease];
	[req setValue:@"1" forHTTPHeaderField:@"icy-metadata"];
	[req setValue:@"no-cache" forHTTPHeaderField:@"Cache-Control"];
	// Shoutcast seams to need this for some reason
	[req setValue:@"iTunes/4.6 (Macintosh; N; PPC)" forHTTPHeaderField:@"User-Agent"];
	conn = [[NSURLConnection connectionWithRequest:req delegate:self] retain];
	if (!conn) {
		[self log: @"Can not open connection to the station. Recording failed\n"];
		[self checkResume];
		[self endRecord];
		return NO;
	}
	icyInterval = 0;
	icyLen = 0;
	icyMark = 0;
	[genre release];
	[title release];
	if (icyData == nil)
		icyData = [[NSMutableData alloc] init];
	else
		[icyData setLength:0];
	genre = title = icyLast = @"";
	dataSize = lastSize = 0;
	return YES;
}
	

-(void) record: (NSTimer *)tim {
	[self deschedule];
	NSCalendarDate *cal = [NSCalendarDate calendarDate];
	/*
	[log setString: @""];
	if (text)
		[text setString: log];
	 */
	if (progress)
		[progress setDoubleValue: 0];
	[self fmtlog:  @"Started recording  at %@\n", [self stringFromDate: cal]];
	if (once) {
		int cday = [cal dayOfWeek];
		cday = cday == 0 ? 6 : cday-1;
		days[cday] = NO;
		[recorder clearDay: cday];
	}
	if (!willResume || !tim) {
		[startDate release];
		startDate = [[NSCalendarDate calendarDate] retain];
	}
	willResume = NO;
	percent = 0;
	inProgress = YES;
	assert (timer == nil);
	timer = [[NSTimer scheduledTimerWithTimeInterval:60
											  target:self selector:@selector(checkRecord:) 
								   userInfo:nil repeats:YES] retain];
	[recorder enableButtons];
	[self connect: url];
}

- (void)connection:(NSURLConnection *)connection didFailWithError:(NSError *)error {
	[self fmtlog: @"Connection failed with error: %@\n", error];
	[self checkResume];
	[self endRecord];
}

-(void)connection:(NSURLConnection *)connection didReceiveResponse:(NSURLResponse *)response {
	[self log: @"Getting a response\n"];
	if ([response isKindOfClass:[NSHTTPURLResponse class]]) {
		NSHTTPURLResponse *u = (NSHTTPURLResponse *)response;
		NSDictionary *d = [u allHeaderFields];
		[self fmtlog: @"HTTP Headers: %@\n", [d description]];
		NSEnumerator *n = [d keyEnumerator];
		NSString *hdr;
		while ((hdr = [n nextObject]) != nil) {
			if ([hdr caseInsensitiveCompare:@"icy-metaint"] == NSOrderedSame) {
				icyInterval = [(NSString *)[d objectForKey:hdr] intValue];
				if (!icyInterval)
					icyInterval = -1;
			} else if ([hdr caseInsensitiveCompare:@"icy-genre"] == NSOrderedSame) {
				[genre release];
				genre = [[d objectForKey:hdr] retain];
			}
		}
		if (icyInterval > 0) {
			[self log: [NSString stringWithFormat: @"Shoutcast/icecast metadata interval is %d\n", icyInterval]];
			if (icyData == nil)
				icyData = [[NSMutableArray alloc] init];
			else
				[icyData setLength:0];
		}
		if ([genre length] > 0) 
			[self fmtlog:@"Shoutcast genre is %@\n", genre];
		
	}
	NSString *mime = [response MIMEType];
	NSRange r = [mime rangeOfString:@"/ogg" options:NSCaseInsensitiveSearch];
	if (r.location != NSNotFound) {
		[self log: @"Detected ogg audio\n"];
		fileExt = @".ogg";
	} else
		fileExt = @".mp3";
}

-(NSURLRequest *)connection:(NSURLConnection *)connection
            willSendRequest:(NSURLRequest *)request
           redirectResponse:(NSURLResponse *)redirectResponse {
	[self fmtlog:@"Connection is redirected to %@\n", [request URL]];
	return request;
}

-(void)openFile: (NSString *)newTitle {
	madeTitle = [newTitle length] == 0;
	NSString *fileName;
	if (madeTitle) {
		NSString *pfx = [customTitle length] > 0 ? customTitle : name;
		NSCalendarDate *cdate = [NSCalendarDate calendarDate];
		fileName = [cdate descriptionWithCalendarFormat:
			[pfx stringByAppendingString:@"_%b_%d_%Y__%I_%M%p"]];
		newTitle = [cdate descriptionWithCalendarFormat:
			[pfx stringByAppendingString:@" - %b %d, %Y %I:%M%p"]];
	} else {
		NSMutableString *fname = [[NSMutableString alloc] initWithString: newTitle];
		[fname autorelease];
		[fname replaceOccurrencesOfString:@"/" withString:@"_" options:0 range:NSMakeRange(0,[fname length])];
		[fname replaceOccurrencesOfString:@"\\" withString:@"_" options:0 range:NSMakeRange(0,[fname length])];
		[fname replaceOccurrencesOfString:@":" withString:@"_" options:0 range:NSMakeRange(0,[fname length])];
		[fname replaceOccurrencesOfString:@"'" withString:@"_" options:0 range:NSMakeRange(0,[fname length])];
		[fname replaceOccurrencesOfString:@"\"" withString:@"_" options:0 range:NSMakeRange(0,[fname length])];
		[fname replaceOccurrencesOfString:@"~" withString:@"" options:0 range:NSMakeRange(0,[fname length])];
		fileName = fname;
	}
	fileName = [fileName stringByAppendingString:fileExt];
	[title release];
	title = [newTitle retain];
		
	NSString *dir = NSHomeDirectory();
	NSFileManager *f = [NSFileManager defaultManager];
	dir = [dir stringByAppendingPathComponent:@"Music"];
	[f createDirectoryAtPath: dir attributes: nil];
	dir = [dir stringByAppendingPathComponent:@"Radio Recorder"];
	[f createDirectoryAtPath: dir attributes: nil];	
	[filePath release];
	filePath = [[dir stringByAppendingPathComponent: fileName] retain];
	[f removeFileAtPath: filePath handler: nil];
	[f createFileAtPath:filePath contents:[NSData data] attributes:nil];
	[self fmtlog: @"Writting data to %@\n", fileName];
	file = [[NSFileHandle fileHandleForWritingAtPath:filePath] retain];
	if (!file)
		[NSException raise:@"FileOpenFailed" format:filePath];
}

- (void)processIcy: (NSString *)hdr {
	[self log: hdr];
	NSRange whole = NSMakeRange(0, [hdr length]);
	for (;;) {
		NSRange eol = [hdr rangeOfString:@"\r\n" options:0 range:whole];
		if (eol.location == NSNotFound) break;
		NSString *line = [hdr substringWithRange: NSMakeRange(whole.location, eol.location-whole.location)];
		NSString *ll = [line lowercaseString];
		if ([ll hasPrefix: @"content-type:"]) {
			NSRange r = [ll rangeOfString:@"/ogg"];
			if (r.location != NSNotFound) {
				[self log: @"Detected ogg from icecast data\n"];
				fileExt = @".ogg";
			}
		} else if ([ll hasPrefix: @"icy-genre:"]) {
			[genre release];
			genre = [[[line substringFromIndex: sizeof("icy-genre:")-1] stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceAndNewlineCharacterSet]] retain];
			
		} else if ([ll hasPrefix: @"icy-metaint:"]) {
			icyInterval = [[line substringFromIndex: sizeof("icy-metaint:")-1] intValue];
			if (!icyInterval) icyInterval = -1;
		}
		whole.length-=eol.location-whole.location+2;
		whole.location=eol.location+2;
	}
}

- (void)changeTrack: (NSString *)icy {
	if ([icy isEqual: icyLast])
		return;
	NSString *newTitle;
	NSRange match = [icy rangeOfString:@"StreamTitle='" options:NSCaseInsensitiveSearch];
	if (match.location == NSNotFound) {
		willResume = YES;
		[NSException raise:@"LostStreamSync" format:@"Lost sync with shoutcast data, disconnecting"];
	}
	[self fmtlog: @"Got shoutcast data: %@\n", icy];
	int start = match.location+match.length;
	NSRange st = [icy rangeOfString:@"';" options:0 range:NSMakeRange(start, [icy length]-start)];
	if (st.location == NSNotFound)
		newTitle = [icy substringFromIndex:start];
	else
		newTitle = [icy substringWithRange:NSMakeRange(start, st.location-start)];
	int len = [icyLast length], nlen = [newTitle length];
	[icyLast release];
	icyLast = [icy retain];
	if ((!keepPartial && !len && nlen) || !splitSongs) return;
	[self closeFile: NO];
	if (keepPartial && !len && [newTitle length] > 0)
		newTitle = [ENDOF stringByAppendingString:newTitle];
	[self openFile: newTitle];
}

- (void)processData: (const char *)p withLength: (int)len {
	if ((icyInterval < 0 || !splitSongs) && file == nil)
		[self openFile: @""];
	for (;;) {
		int copyLen;
		if (icyLen) {
			copyLen = MIN(icyLen, len);
			[icyData appendBytes:p  length:copyLen];
			icyLen-=copyLen;
			len-=copyLen;
			p+=copyLen;
			if (!icyLen) {
				char b = 0;
				[icyData appendBytes:&b length:1];
				NSString *icy =  [self langString:[icyData bytes] withLength:[icyData length]];
				[self changeTrack: icy];
			}
		}
		if (!len) break;
		int nextOff = (dataSize+icyInterval-1+icyMark)/icyInterval*icyInterval;
		copyLen = icyInterval < 0 || dataSize == 0 ? len: MIN(len, nextOff-dataSize);
		if (copyLen) {
			if (file)
				[file writeData: [NSData dataWithBytesNoCopy:(void *)p length:copyLen freeWhenDone:NO]];
			dataSize+=copyLen;
			len-=copyLen;
			p+=copyLen;
			icyMark = 0;
		}
		if (!len) break;
		[icyData setLength:0];
		icyLen = 16*(unsigned char)(*p++);
		len--;
		icyMark = 1;
	}
}
	

- (void)connection:(NSURLConnection *)connection didReceiveData:(NSData *)data {
	int len = [data length];
	const char *p = (const char *)[data bytes];
	NS_DURING
		if (!icyInterval) {
			icyInterval = -1;
			int copyLen = MIN(len, ICY_MAXHEADER-[icyData length]);
			[icyData appendBytes: p length: copyLen]; p+=copyLen; len-=copyLen;
			int hdrLen = [icyData length];
			if (hdrLen >= ICY_MAXHEADER) {
				const char *hdr = (const char *)[icyData bytes], *endHdr;
				if (!strncmp(hdr, "ICY", 3) && (endHdr = strstr(hdr, "\r\n\r\n"))) {
					[self processIcy:  [self langString:hdr withLength:endHdr-hdr+2]];
					[self processData: endHdr+4 withLength: hdrLen-(endHdr-hdr+4)];
				} else {
					[self processData: hdr withLength: hdrLen];
				}
			}
			if (!len) NS_VOIDRETURN;
		}
		[self processData: p withLength: len];
	NS_HANDLER
		[self fmtlog: @"File access error: %@\n", localException];
		[self endRecord];
	NS_ENDHANDLER
}

-(void)connectionDidFinishLoading:(NSURLConnection *)connection {
	if (dataSize == 0 && [icyData length] > 0) {
		// Check for a PLS or m3u file
		char zero = 0;
		[icyData appendBytes:&zero length:1];
		NSString *s = [NSString stringWithUTF8String:[icyData bytes]];
		if (s == nil) goto doClose;
		NSRange st = [s rangeOfString:@"http://" options:NSCaseInsensitiveSearch];
		if (st.location == NSNotFound) goto doClose;
		NSRange en = [s rangeOfCharacterFromSet:[NSCharacterSet controlCharacterSet] options: 0 range: NSMakeRange(st.location, [s length]-st.location)];
		if (en.location == NSNotFound) en.location = [s length];
		st.length = en.location - st.location;
		[self log: @"Redirecting to the first target in PLS or M3U playlist\n"];
		[icyData setLength:0];
		[self connect: [s substringWithRange:st]];
		return;
	}
doClose:;
	[self log: @"Connection closed by the station\n"];
	[self checkResume];
	[self endRecord];
}

-(void) checkRecord: (NSTimer *)tim {
	int mn = [[NSDate date] timeIntervalSinceDate: startDate]/60;
	[self log: [NSString stringWithFormat: @"Data size is %d at %d minutes\n", dataSize, mn]];
	if (dataSize == lastSize) {
		[self log: @"Connection stalled, ending recording\n"];
		[self checkResume];
		[self endRecord];
		return;
	}
	lastSize = dataSize;
	if (progress)
		[progress setDoubleValue: percent = 100.0*mn/duration];
	if (mn >= duration) {
		[self fmtlog: @"Recording complete (%d minutes), closing connection\n", duration];
		[self endRecord];
	}
}

-(void)addToTunes  {	
	NSDictionary *err=nil;
	if (!addScript) {
		NSString *scriptPath = [[NSBundle bundleForClass:[self class]] pathForResource: @"addTune" ofType: @"scpt"];
		NSURL *scriptURL = [NSURL fileURLWithPath: scriptPath];
		addScript = [[NSAppleScript alloc] initWithContentsOfURL: scriptURL error: &err];
		if (!addScript) 
			goto error;
	}
	NSString *artist = customArtist, *album=[customAlbum length] > 0 ? customAlbum : name, *stitle=title;
	if (splitTitle && !madeTitle) {
		NSString *tpfx = @"";
		if ([stitle hasPrefix: STARTOF]) {
			tpfx = STARTOF;
			stitle = [stitle substringFromIndex:[STARTOF length]];
		}
		if ([stitle hasPrefix: ENDOF]) {
			tpfx = [tpfx stringByAppendingString: ENDOF];
			stitle = [stitle substringFromIndex:[ENDOF length]];
		}
		NSString *parts[3] = {nil};
		int np=0;
		NSScanner *scn = [NSScanner scannerWithString:stitle];
		NSCharacterSet *nn = [[NSCharacterSet decimalDigitCharacterSet] invertedSet];
		[scn setCharactersToBeSkipped:[NSCharacterSet characterSetWithCharactersInString:@""]];
		while (np < 3 && scn != nil && ![scn isAtEnd]) {
			NSString *p;
			if (![scn scanUpToString:@" - " intoString:&p])
				break;
			[scn scanString:@" - " intoString:nil];
			p = [p stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			//if ([p hasPrefix:@"~"]) continue;
			NSRange r = [p rangeOfCharacterFromSet:nn];
			if (r.location == NSNotFound) continue;
			parts[np++] = p;
		}
		switch (np) {
			case 1:
				stitle = parts[0];
				break;
			case 2:
				artist = parts[0];
				stitle = parts[1];
				break;
			case 3:
				artist = parts[0];
				album = parts[1];
				stitle = parts[2];
				break;
		}
		stitle = [tpfx stringByAppendingString:stitle];
	}
	
    NSAppleEventDescriptor *arguments = [[NSAppleEventDescriptor alloc] initListDescriptor];
	int pos = 0;
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithString: filePath] atIndex: ++pos];
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithString: playList] atIndex: ++pos];
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithString: stitle] atIndex: ++pos];
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithString: album] atIndex: ++pos];
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithString: artist] atIndex: ++pos];
	NSString *gName = [customGenre length] > 0 ? customGenre : genre;
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithString: gName] atIndex: ++pos];
    [arguments insertDescriptor: [NSAppleEventDescriptor descriptorWithInt32: recCount >= 0 ? recCount : 1<<20] atIndex: ++pos];
	NSAppleEventDescriptor *result = [addScript callHandler: @"add_tune" withArguments: arguments errorInfo: &err];
	[arguments release];
	if (!result) {
error:;
		[self fmtlog: @"Error adding to playlist: %@\n", err];
		if (conn)
			[NSException raise:@"FileAddFailed" format:@"Error adding the track to iTunes"];
	} else
		[self fmtlog:@"Added track to playlist %@\n", playList];
}

-(void)closeFile: (BOOL)isLast {
	if (file == nil) return;
	int sz = [file offsetInFile];
	if (icyInterval > 0 && sz > icyInterval)
		[file truncateFileAtOffset:sz-icyInterval]; // Prevent potential cross-fade
	[file synchronizeFile];
	[file release];
	file = nil;
	NSFileManager *fm = [NSFileManager defaultManager];
	if (isLast) {
		if (!keepPartial) {
			[self log: @"Deleting the last (incomplete) shoutcast song\n"];
			[fm removeFileAtPath:filePath handler:nil];
			return;
		}
		[title autorelease];
		title = [[STARTOF stringByAppendingString:title] retain];
		NSString *dir = [filePath stringByDeletingLastPathComponent];
		NSString *base = [STARTOF stringByAppendingString: [filePath lastPathComponent]];
		NSString *newPath = [dir stringByAppendingPathComponent:base];
		[fm removeFileAtPath:newPath handler:nil];
		if ([fm movePath:filePath toPath:newPath handler:nil]) {
			[filePath release];
			filePath = [newPath retain];
		} else
			[self fmtlog:@"Couldn't rename %@ to %@\n", filePath, newPath];
	}
		
	if (sz < 256*1024) {
		[self fmtlog: @"Data size is tiny (%d bytes), deleting song\n", sz];
		[fm removeFileAtPath:filePath handler:nil];
	} else {
		[self log: [NSString stringWithFormat: @"File size is %d bytes\n", sz]];
		if ([playList length] > 0)
			[self addToTunes];
	}
}

-(void)endRecord {
	if (!conn) return;
	[conn cancel];
	[conn autorelease];
	conn = nil;
	[self deschedule];
	inProgress = NO;
	[self closeFile: !madeTitle];
	[self log: [NSString stringWithFormat:@"Recording ended at %@\n", [self stringFromDate: [NSCalendarDate calendarDate]]]];
	[self schedule];
	[recorder enableButtons];
	[recorder storeRecList];
}

-(void)checkResume {
	if (reconnect) {
		int mn = [[NSDate date] timeIntervalSinceDate: startDate]/60;
		if (duration - mn > 15)
			willResume = YES;
	}
}

-(NSComparisonResult)compareTo: (Recording *)otr {
	return [name caseInsensitiveCompare: otr->name];
}
@end
