//
//  UIValueManager.m
//  RadioRecorder
//
//  Created by Oleg Kibirev on Sat May 15 2004.
//  Copyright (c) 2004 __MyCompanyName__. All rights reserved.
//

#import "UIValueManager.h"
#import <objc/objc-runtime.h>


@implementation UIValueManager
-(id)initWithValInfo: (UIValInfo *)n ofCount: (size_t)cnt {
	self = [super init];
	info = n;
	count = cnt;
	return self;
}

-(void)initObject: (id)obj {
	int i;
	Class cls = [obj class];
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)obj+v->ivar_offset;
		const char *t = v->ivar_type;
		if (!strcmp(t, @encode(BOOL)))
			*(BOOL *)p = n->defNum;
		else if (!strcmp(t, @encode(int)))
			*(int *)p = n->defNum;
		else if (!strcmp(t, @encode(double)))
			*(double *)p = n->defNum;
		else if (!strcmp(t, "@\"NSString\"")) {
			const char *s = n->defStr ? n->defStr : "";
			*(NSString **)p = [[NSString alloc] initWithUTF8String:s];
		} else
			NSAssert1(NO, @"Unknown type: %s", t);
	}
}

-(void)deallocObject: (id)obj {
	Class cls = [obj class];
	int i;
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)obj+v->ivar_offset;
		const char *t = v->ivar_type;
		if (!strcmp(t, "@\"NSString\""))
			[*(NSString **)p release];
	}
}

-(void)copyObject: (id)dst fromObject: (id)src {
	Class cls = [dst class];
	int i;
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)dst+v->ivar_offset, *q = (char *)src+v->ivar_offset;
		const char *t = v->ivar_type;
		if (!strcmp(t, @encode(BOOL)))
			*(BOOL *)p = *(BOOL *)q;
		else if (!strcmp(t, @encode(int)))
			*(int *)p = *(int *)q;
		else if (!strcmp(t, @encode(double)))
			*(double *)p = *(double *)q;
		else if (!strcmp(t, "@\"NSString\"")) {
			[*(NSString **)q retain];
			[*(NSString **)p release];
			*(NSString **)p = *(NSString **)q;
		} else
			NSAssert1(NO, @"Unknown type: %s", t);
	}
}

-(void)encodeObject: (id)obj withCoder: (NSCoder *)coder {
	int i;
	Class cls = [obj class];
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		if (n->flags & UIVAL_TRANSIENT) continue;
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)obj+v->ivar_offset;
		const char *t = v->ivar_type;
		NSString *key = [NSString stringWithUTF8String:n->varName];
		if (!strcmp(t, @encode(BOOL)))
			[coder encodeBool:*(BOOL *)p forKey:key];
		else if (!strcmp(t, @encode(int)))
			[coder encodeInt:*(int *)p forKey:key];
		else if (!strcmp(t, @encode(double)))
			[coder encodeDouble:*(double *)p forKey:key];
		else if (!strcmp(t, "@\"NSString\""))
			[coder encodeObject: *(NSString **)p forKey: key];
		else
			NSAssert1(NO, @"Unknown type: %s", t);
	}
}	

-(void)decodeObject: (id)obj withCoder: (NSCoder *)coder {
	[self initObject: obj];
	Class cls = [obj class];
	int i;
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		if (n->flags & UIVAL_TRANSIENT) continue;
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)obj+v->ivar_offset;
		const char *t = v->ivar_type;
		NSString *key = [NSString stringWithUTF8String:n->varName];
		if (![coder containsValueForKey:key]) continue;
		if (!strcmp(t, @encode(BOOL)))
			*(BOOL *)p = [coder decodeBoolForKey:key];
		else if (!strcmp(t, @encode(int)))
			*(int *)p = [coder decodeIntForKey:key];
		else if (!strcmp(t, @encode(double)))
			*(double *)p = [coder decodeDoubleForKey:key];
		else if (!strcmp(t, "@\"NSString\"")) {
			[*(NSString **)p release];
			*(NSString **)p = [[coder decodeObjectForKey:key] retain];
		} else
			NSAssert1(NO, @"Unknown type: %s", t);
	}
}

-(void)loadObject: (id)obj toUI:(id)ui {
	Class cls = [obj class], uc = [ui class];
	int i;
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		if (!n->uiName) continue;
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)obj+v->ivar_offset;
		const char *t = v->ivar_type;
		Ivar u = class_getInstanceVariable(uc, n->uiName);
		NSAssert1(u != NULL, @"UI Instance variable not found", n->uiName);
		NSControl *ctl = *(NSControl **)((char *)ui+u->ivar_offset);
		if ([ctl isKindOfClass:[NSButton class]]) {
			NSAssert1(!strcmp(t, @encode(BOOL)), @"Value for button should be BOOL, is actually %s", t);
			[(NSButton *)ctl setState: *(BOOL *)p];
		} else if (!strcmp(t, @encode(int)))
			[ctl setIntValue:*(int *)p];
		else if (!strcmp(t, @encode(double)))
			[ctl setDoubleValue:*(double *)p];
		else if (!strcmp(t, "@\"NSString\""))
			[ctl setStringValue:*(NSString **)p];
		else
			NSAssert1(NO, @"Unknown type: %s", t);
	}
}

-(void)saveUI: (id)ui toObject: (id)obj {
	Class cls = [obj class], uc = [ui class];
	int i;
	for (i = 0; i < count; i++) {
		UIValInfo *n = &info[i];
		if (!n->uiName) continue;
		Ivar v = class_getInstanceVariable(cls, n->varName);
		NSAssert1(v != NULL, @"Instance variable %s not found", n->varName);
		char *p = (char *)obj+v->ivar_offset;
		const char *t = v->ivar_type;
		Ivar u = class_getInstanceVariable(uc, n->uiName);
		NSAssert1(u != NULL, @"UI Instance variable not found", n->uiName);
		NSControl *ctl = *(NSControl **)((char *)ui+u->ivar_offset);
		if ([ctl isKindOfClass:[NSButton class]]) {
			NSAssert1(!strcmp(t, @encode(BOOL)), @"Value for button should be BOOL, is actually %s", t);
			*(BOOL *)p = [(NSButton *)ctl state] == NSOnState;
		} else if (!strcmp(t, @encode(int)))
			*(int *)p = [ctl intValue];
		else if (!strcmp(t, @encode(double)))
			*(double *)p = [ctl doubleValue];
		else if (!strcmp(t, "@\"NSString\"")) {
			NSString *s = [ctl stringValue];
			if (!(n->flags & UIVAL_NOTRIM))
				s = [s stringByTrimmingCharactersInSet:[NSCharacterSet whitespaceCharacterSet]];
			[s retain];
			[*(NSString **)p release];
			*(NSString **)p = s;
		} else
			NSAssert1(NO, @"Unknown type: %s", t);
	}
}
	
@end
